#!/bin/bash

args=("$@")

arg_len=${#args[@]}

if [ "$arg_len" -lt "2" ]; then
    echo "Must pass serial number as argument #1 and MAC address as argument #2"
    exit -1
fi

serial=${args[0]}
mac_addr=${args[1]}

bad_ip=true

while [[ $bad_ip == true ]]; do
    echo -n "Plese enter the IP address of the control point (bone): "
    read cp_ip
    echo -n "Please confirm the IP address: "
    read cp_ip_confirm

    fmt=`echo $cp_ip | grep -Eo '^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$'`

    if [[ "$cp_ip" != "$cp_ip_confirm" ]]; then
        echo "IP addresses do not match, please try again."
        echo ""
    elif [[ "$fmt" == "" ]]; then
        echo "IP addresses must be in the form xxx.xxx.xxx.xxx, please try again."
        echo ""
    else
        bad_ip=false
    fi
done

echo ""

bad_ip=true

while [[ $bad_ip == true ]]; do
    echo -n "Plese enter the IP address of the renderer (brain): "
    read renderer_ip
    echo -n "Please confirm the IP address: "
    read renderer_ip_confirm

    fmt=`echo $renderer_ip | grep -Eo '^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$'`

    if [[ "$renderer_ip" != "$renderer_ip_confirm" ]]; then
        echo "IP addresses do not match, please try again."
        echo ""
    elif [[ "$fmt" == "" ]]; then
        echo "IP addresses must be in the form xxx.xxx.xxx.xxx, please try again."
        echo ""
    else
        bad_ip=false
    fi
done

echo ""

rm -rf /home/boulder/.ssh/known_hosts

echo "Setting serial number of Control Point..."
sshpass -p 9pwStVmptc8cLcgZ ssh -oStrictHostKeyChecking=no root@$cp_ip -p4422 "mount -o remount,rw / && hostname 2120-controlpoint-$serial && echo \"2120-controlpoint-$serial\" > /etc/hostname && sync"

return_code=$?

if [[ $return_code != 0 ]]; then
    echo "Unable to set serial number of Control Point: $return_code"
    echo "Aborting."
    exit $return_code
fi

echo ""
echo "Done!"
echo ""

echo "Setting serial number of Renderer..."
sshpass -p 9pwStVmptc8cLcgZ ssh -oStrictHostKeyChecking=no root@$renderer_ip -p4422 "mount -o remount,rw / && hostname 2120-renderer-$serial && echo \"2120-renderer-$serial\" > /etc/hostname && sync"
sshpass -p 9pwStVmptc8cLcgZ ssh -oStrictHostKeyChecking=no root@$renderer_ip -p4422 "cat /home/root/sim/skins/realskins/default/config-menu | sed -e \"s/upnp_friendly_name.*/upnp_friendly_name Default Room : 2120 D\/A [$serial]/\" > /home/root/sim/skins/realskins/default/config-menu.tmp && mv /home/root/sim/skins/realskins/default/config-menu.tmp /home/root/sim/skins/realskins/default/config-menu && sync"

return_code=$?

if [[ $return_code != 0 ]]; then
    echo "Unable to set serial number of Renderer: $return_code"
    echo "Aborting."
    exit $return_code
fi

echo ""
echo "Done!"
echo ""

echo "Setting MAC address of Renderer..."

# First, check if the renderer has a MAC set yet or not
sshpass -p 9pwStVmptc8cLcgZ ssh -oStrictHostKeyChecking=no root@$renderer_ip -p4422 "cat /writeable/interfaces | grep hwaddress"
return_code=$?

# MAC was not set
if [[ $return_code != 0 ]]; then
    sshpass -p 9pwStVmptc8cLcgZ ssh -oStrictHostKeyChecking=no root@$renderer_ip -p4422 "sed -i \"24 i\
\ \ \ \ \ \ \ \ hwaddress ether $mac_addr\" \/writeable\/interfaces && cp /writeable/interfaces /writeable/interfaces_dhcp && cat /writeable/interfaces_dhcp | sed -e \"s/iface eth0 inet dhcp/iface eth0 inet static\n\taddress 192.168.0.200\n\tnetmask 255.255.255.0\n\tnetwork 192.168.0.0\n\tbroadcast 192.168.0.255\n\tgateway 192.168.0.1/\" > /writeable/interfaces_static && sync"
else
    # Replace existing MAC
    sshpass -p 9pwStVmptc8cLcgZ ssh -oStrictHostKeyChecking=no root@$renderer_ip -p4422 "sed -i \"/hwaddress ether/c\
\ \ \ \ \ \ \ \ hwaddress ether $mac_addr\" \/writeable\/interfaces && cp /writeable/interfaces /writeable/interfaces_dhcp && cat /writeable/interfaces_dhcp | sed -e \"s/iface eth0 inet dhcp/iface eth0 inet static\n\taddress 192.168.0.200\n\tnetmask 255.255.255.0\n\tnetwork 192.168.0.0\n\tbroadcast 192.168.0.255\n\tgateway 192.168.0.1/\" > /writeable/interfaces_static && sync"
fi

return_code=$?

if [[ $return_code != 0 ]]; then
    echo "Unable to set MAC address of Renderer: $return_code"
    echo "Aborting."
    exit $return_code
fi

echo ""
echo "Done!"
echo ""
echo "New serial number:        $serial"
echo "New renderer MAC address: $mac_addr"
echo ""
echo ""

exit 0

