#!/usr/bin/python
import platform
import time
from math import cos, pi
from collections import deque

PWM_WHITE="/sys/class/pwm/pwmchip0/pwm0/duty_cycle"
PWM_RED="/sys/class/pwm/pwmchip0/pwm1/duty_cycle"

LED_PERIOD = 1000
PWM_OFFSET = 850
PWM_DUTIES_LEN = 160

PWM_DUTIES = [PWM_OFFSET] * PWM_DUTIES_LEN
for i in range(len(PWM_DUTIES)):
    PWM_DUTIES[i] = int((PWM_OFFSET)/2 * cos(2*pi*(i /float(PWM_DUTIES_LEN))) + (PWM_OFFSET)/2)

"""
for i in range(0, 80):
    PWM_DUTIES[i] = PWM_DUTIES[i] - (7*(i))

for i in range(80, 160):
    PWM_DUTIES[i] =  0  + (7*(i-79))
"""

class LedState:
    def __init__(self):
        self.pwm_duty_white = 0
        self.pwm_duty_red = 0 
        self.led_state = "standby"
        self.previous_state = "standby"
        self.blink_state = False
        self.switch_time = time.time()
        self.state_queue = deque([], 9)

    def set_white(self, value):
        with open(PWM_WHITE, "w") as pwm_white:
            pwm_white.write(str(value))

    def set_red(self, value):
        with open(PWM_RED, "w") as pwm_red:
            pwm_red.write(str(value))

    
    def led_thread(self):
        self.running = True
        while self.running:
            if 'arm' not in platform.machine():
                return
            self.get_state()
            if self.led_state == "standby":
                if time.time() - self.switch_time > 0.03:
                    if self.previous_state == "transient_error":
                        time.sleep(1)
                    self.pwm_duty_white = (self.pwm_duty_white + 1) % (PWM_DUTIES_LEN)
                    self.set_white(PWM_DUTIES[self.pwm_duty_white])
                    self.set_red(1000)
                    self.switch_time = time.time()
            elif self.led_state == "on":
                if time.time() - self.switch_time > 0.3:
                    if self.previous_state == "transient_error":
                        time.sleep(1)
                    self.set_white(0)
                    self.set_red(1000)
                    self.switch_time = time.time()
            elif self.led_state == "fatal_error":
                self.set_white(1000)
                if time.time() - self.switch_time > 0.3:
                    if self.blink_state == True:
                        self.set_red(0)
                        self.blink_state = False
                        self.switch_time = time.time()
                    else:
                        self.set_red(1000)
                        self.blink_state = True
                        self.switch_time = time.time()
            elif self.led_state == "transient_error":
                self.set_white(1000)
                self.set_red(0)
            elif self.led_state == "turning_on":
                if time.time() - self.switch_time > 0.5:
                    if self.blink_state == True:
                        self.set_white(0)
                        self.set_red(1000)
                        self.blink_state = False
                    else:
                        self.set_white(1000)
                        self.set_red(0)
                        self.blink_state = True
                    self.switch_time = time.time()
            elif self.led_state == "mains_critical":
                if time.time() - self.switch_time > 0.03:
                    self.pwm_duty_white = (self.pwm_duty_white + 3) % (PWM_DUTIES_LEN)
                    self.set_white(PWM_DUTIES[self.pwm_duty_white])
                    self.set_red(1000)
            elif self.led_state == "mains_warning":
                self.set_white(1000)
                self.set_red(0)
            elif self.led_state == "updating":
                if time.time() - self.switch_time > 1.0:
                    if self.blink_state == True:
                        self.set_white(0)
                        self.set_red(1000)
                        self.blink_state = False
                    else:
                        self.set_white(1000)
                        self.set_red(1000)
                        self.blink_state = True
                    self.switch_time = time.time()
            elif self.led_state == "update_failure":
                if time.time() - self.switch_time > 1.0:
                    if self.blink_state == True:
                        self.set_white(1000)
                        self.set_red(0)
                        self.blink_state = False
                    else:
                        self.set_white(1000)
                        self.set_red(1000)
                        self.blink_state = True
                    self.switch_time = time.time()
            else:
                print "ERROR: invalid state: " + self.led_state
	    time.sleep(0.03)

    def set_state(self, state):
        if state not in self.state_queue:
            self.state_queue.appendleft(state)

    def get_state(self):
        self.previous_state = self.led_state
        if len(self.state_queue):
            self.led_state = self.state_queue.pop()
        if self.previous_state != self.led_state:
            self.switch_time = 0

    def queued(self, state):
        return state in self.state_queue

if __name__ == '__main__':
    ledstate = LedState()
    #ledstate.set_state("error")
    ledstate.led_thread()
