import time

import log_commands


def get_event_lines(lines):
    event_lines = []
    for line in lines:
        if 'EVENT' in line:
            event = {}
            attributes = line.split(':')
            event['data'], hours = attributes[0].split('T')
            minutes = attributes[1]
            seconds = attributes[2]
            event['time'] = float(seconds) + (int(minutes)*60) + (int(hours)*3600)
            event['priority'] = attributes[3]
            event['file'] = attributes[4]
            event['function'] = attributes[5]
            event['command'] = attributes[7]
            event['params'] = attributes[8]
            event_lines.append(event)
    return event_lines


def readback(log_file_name):
    command_handler = log_commands.CommandHandler()

    with open(log_file_name,'r') as log_fp:

        event_lines = get_event_lines(log_fp.readlines())
    for i in range(len(event_lines)-1):
        event = event_lines[i]
        next_event = event_lines[i+1]
        command_handler.RunCommand(event['command'], event['params'])
        #TODO What if the day changes
        wait_time = next_event['time'] - event['time']
        time.sleep(wait_time)


    command_handler.RunCommand(event_lines[-1]['command'], event_lines[-1]['params'])
    #log_commands.commands[event_lines[-1]['command']](event_lines[-1]['params'])


def compare_logs(amp_file_name, control_file_name, use_startup=False):

    with open(amp_file_name,'r') as amp_fp:
        amp_event_lines = get_event_lines(amp_fp.readlines())
        
    with open(control_file_name,'r') as control_fp:
        control_event_lines = get_event_lines(control_fp.readlines())

    #filter out events that came from SetupServices
    if not use_startup:
        #TODO do something different
        control_event_lines = control_event_lines[43:]   
        #amp_event_lines = amp_event_lines[43:]   
    
    print 'Amp log length: ', len(amp_event_lines)
    print 'Micromedia log length: ', len(control_event_lines)
    matching = True
    for i in range(min(len(amp_event_lines),len(control_event_lines))):
        amp_event = amp_event_lines[i]
        control_event = control_event_lines[i]

        match = amp_event['command'] == control_event['command'] and amp_event['params'] == control_event['params']
        if match:
            print "EVENT MATCH"

        else:
            matching = False
            print "ERROR log mismatch on amp event: ", amp_event['command'], ':', amp_event['params']
            print "Does not match:                  ", control_event['command'], ':', control_event['params']
            print '\n\n'
    if len(amp_event_lines) != len(control_event_lines):
        print "ERROR lengths did not match"
	matching = False

    return matching


if __name__ == '__main__':
    amp_filename = 'logout/amp.log'
    control_filename = 'logout/micromedia.log'
    print compare_logs(amp_filename, control_filename)
    #readback(amp_filename)
