import os
os.chdir(os.path.abspath(os.path.dirname(__file__)))
import platform
import sys
sys.path.append('./website')
if 'arm' in platform.machine():
    PC=False
    sys.path.append('./build_arm')
else:
    PC=True
    sys.path.append('./build')

import threading
import argparse
import logging

import pylogger
from pylogger import bllogger
import libamplifier_lib as amp

import python_settings
import main_page
import rest_api



def fatal_check():
    fatal_errors = amplifier_setting.check_fatal()
    if fatal_errors:
        bllogger('debug', 'fatal errors')
        print "###############"
        print 'FATAL ERRORS'
        print "###############"
        amplifier_setting.fatal_shutdown = True
        amplifier_setting.led_state.set_state('fatal_error')
        amplifier_setting.mute_state.set_state('off')
        diag = False
        while not diag:
            diag = key_handle.check_diag()
        amplifier_setting.clear_fatals()
        amplifier_setting.clear_errors()
    amplifier_setting.fatal_shutdown = False


def startup():
    bllogger('info', 'Startup about to start flask init')
    main_page.app.register_blueprint(main_page.main, url_prefix='/main')
    main_page.app.register_blueprint(rest_api.api_point, url_prefix='/api')
    main_page.app.config.update(
            APPLICATION_ROOT=__file__+'/website',
            template_folder=os.path.join(os.path.dirname(os.path.abspath(__file__)),'website/template'),
            static_folder=os.path.join(os.path.dirname(os.path.abspath(__file__)),'website/static')
            )
    log = logging.getLogger('werkzeug')
    log.setLevel(logging.ERROR)
    rest_api.init_rest_api(main_page.app)
    rest_api.register_headers(main_page.app)
    port=5000 if PC else 80
    try:
        flask_thread = threading.Thread(target=main_page.app.run, kwargs={'threaded':'True', 'host':'0.0.0.0', 'port':port})
        #flask_thread = threading.Thread(target=main_page.app.run, kwargs={'debug':True, 'use_reloader':False, 'threaded':'True', 'host':'0.0.0.0', 'port':port})
        flask_thread.start()
    except Exception as e:
        bllogger('err', "Recieved error when spawning flask thread")
        bllogger('err', e)
    #main_page.app.run(debug=True, threaded=True ,port=port)
    key_handle.get_gpio_fd()
    gpio_handle.relays_off();
    amplifier_setting.find_id()
    amplifier_setting.init_settings()
    fatal_check()
    


def python_poll():
    #move into process state
    key_handle.process()
    rtn = amplifier_setting.process_state()
    if not rtn:
	fatal_check()


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--config', default='config',help="Name of config file to use")
    args = parser.parse_args()
    try:
        global running
        pylogger.setup('amplifier', 'logout')
        amp.CreateSubsystems()
        amp.SetConfigFile(args.config)
        python_settings.init_globals()
        global amplifier_setting
        global key_handle
        global gpio_handle
        amplifier_setting = python_settings.amplifier_setting
        key_handle = python_settings.key_handle
        gpio_handle = python_settings.gpio_handle
        amplifier_setting.add_key_handle(key_handle)
        bllogger('info', 'Done setting up services')
        startup()
    except KeyboardInterrupt:
        bllogger('err', 'Received keyboard interupt exiting...')
        amp.EmergencyShutdown();
        exit(0);
    while True:
        try:
            python_poll()
        except KeyboardInterrupt:
            bllogger('err', 'Received keyboard interupt exiting...')
            amp.EmergencyShutdown();
            break
