from flask import Flask, render_template, Blueprint, url_for, redirect, request, jsonify
import requests
from flask_bootstrap import Bootstrap
from pylogger import bllogger
import time
main = Blueprint('main', __name__ )


app = Flask(__name__)
Bootstrap(app)


@app.route('/')
def main_display():
    api_endpoints = [
            {'url': 'api/error/fatal_errors', 'name': 'fatal_errors'},
            {'url': 'api/error/transient_errors', 'name': 'trans_errors'},
            {'url': 'api/room/room_name', 'name': 'room_name'},
            {'url': 'api/temperature/temperature', 'name': 'temperature'},
            {'url': 'api/update/update_available', 'name': 'update_available'},
            {'url': 'api/update/update_progress', 'name': 'update_progress'},
            {'url': 'api/update/update_in_progress', 'name': 'update_in_progress'},
            {'url': 'api/version/version', 'name': 'software_version'},
            {'url': 'api/version/model_version', 'name': 'model_version'},
            {'url': 'api/mains/mains', 'name': 'mains'},
            {'url': 'api/mains/mains_state', 'name': 'mains_state'},
            {'url': 'api/power/power_switch', 'name': 'power_state'},
            {'url': 'api/global/serial_number', 'name': 'serial_number'}

    ]
    responses = {}
    for endpoint in api_endpoints:
        connecting = True
        retry_count = 0
        while connecting:
            try:
                r = requests.get(request.url_root+endpoint['url'], timeout=1)
                responses[endpoint['name']] = r.json()[endpoint['name']]
                if r.status_code == 200:
                    connecting = False
            except Exception as e:
                bllogger('err', "got exception when connecting to the %s api" % endpoint['name'])
                bllogger('err', '%s' %  e)
                responses[endpoint['name']] = ''
                time.sleep(0.5)
            retry_count += 1
            if retry_count >= 3:
                connecting = False



    return render_template('main_page.html',
                            responses = responses,                    
			    power_link= url_for('.power_switch'),
                            swagger_link = url_for('.swagger'),
                            update_link = url_for('.start_update'),
                            upload_link = url_for('.upload_logs')
			    )


@app.route('/', methods=['POST'])
def room_name_post():
    room_name = request.form['room_name']
    try:
        r = requests.post(request.url_root+'api/room/room_name', data={'room_name':room_name}, timeout=1)
    except Exception as e:
        print "got exception when connecting to room_name post api"
        print e
    if r.status_code is not 200:
        print "ERROR: Could not post the new room name"
    print r.status_code
    print room_name
    return redirect(url_for('.main_display'))

@app.route('/ajax/temperature')
def get_temperature():
    try:
        r = requests.get(request.url_root+'api/temperature/temperature', timeout=1)
        temp = r.json()['temperature']
    except Exception as e:
        print "got exception when connecting to temperature api"
        print e
        temp = 'NA'
    return jsonify({'temp':temp})

@app.route('/ajax/mains')
def get_mains():
    try:
        r = requests.get(request.url_root+'api/mains/mains_state', timeout=1)
        mains = r.json()['mains_state']
    except Exception as e:
        print "got exception when connecting to mains_state api"
        print e
        mains = 'NA'
    return jsonify({'mains_state':mains})


@app.route('/ajax/fatal_errors')
def get_fatal_errors():
    try:
        r = requests.get(request.url_root+'api/error/fatal_errors', timeout=1)
        fatal_errors = r.json()['fatal_errors']
    except Exception as e:
        print "got exception when connecting to fatal_errors  api from ajax endpoint"
        print e
        fatal_errors = {}
    return jsonify({'errors': fatal_errors})


@app.route('/ajax/trans_errors')
def get_trans_errors():
    try:
        r = requests.get(request.url_root+'api/error/transient_errors', timeout=1)
        trans_errors = r.json()['trans_errors']
    except Exception as e:
        print "got exception when connecting to transient_errors  api"
        print e
        trans_errors = {}
    return jsonify({'errors': trans_errors})

@app.route('/ajax/power_on')
def get_power_on():
    try:
        r = requests.get(request.url_root+'api/power/power_switch', timeout=1)
        power_state = r.json()['power_state']
    except Exception as e:
        print "got exception when connecting to power_switch  api"
        print e
        power_state = {}
    return jsonify({'power_state': power_state})

@app.route('/debug')
def swagger():
    return render_template('index.html',
                            url_root = request.url_root)

@app.route('/other/power')
def power_switch():
    print 'power_switch'
    try:
        requests.put(request.url_root+'api/power/power_switch', timeout=1)
    except Exception as e:
        print "got exception when connecting to the power_switch api"
        print e
    return redirect(url_for('.main_display'))


@app.route('/update/start')
def start_update():
    print "start update endpoint"
    try:
        requests.put(request.url_root+'api/update/update_available', timeout=1)
    except Exception as e:
        print "got exception when connecting to the update_avialable api"
        print e
    return redirect(url_for('.main_display'))



@app.route('/log/upload')
def upload_logs():
    try:
        requests.put(request.url_root+'api/log/upload', timeout=1)
    except Exception as e:
        print "got exception when connecting to the upload_log api"
        print e
    return redirect(url_for('.main_display'))
